#!/bin/sh
#########################################################################
#rsyncbackup.sh
#Backup Rsync Script
#by A. Laub 
#andreas@laub-home.de
#
#License:
#This program is free software: you can redistribute it and/or modify it
#under the terms of the GNU General Public License as published by the
#Free Software Foundation, either version 3 of the License, or (at your option)
#any later version.
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
#or FITNESS FOR A PARTICULAR PURPOSE.
#########################################################################

#Load the Pathes
PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin
export PATH

#set the variables:
#Logfile
LOGFILE="/var/log/backup.log"

#Backup Path 
#WARNING: No / at end of path
BACKUPPATH="/srv/samba/DigiCam /srv/samba/MyOffice /srv/samba/Sicherung"

#Exclude Files (for every exclude you need --exclude: "--exclude *.jpg --exclude *.txt)
#EXCLUDE="--exclude *.tc"

#Backup Target 
#for Rsync Daemon: Modulename 
#for Rsync over SSH: Backup path on target system
BACKUPTARGET="Backup"
#BACKUPTARGET="/srv/samba/"

#User
RSUSER="root"

#Target Host
RSTARGET="192.168.52.240"

#E-Mail Address for Statusmail
MAILADRESS="support@laub-home.de"

#Senders E-Mail Adress
FROM="`cat /etc/hostname`@laub-home.de"

#Nagios logfile for check_backuplog.sh
NAGIOSLOGFILE="/var/log/check_backup.log"


#Enable functions:
#Mail Notification (yes/no)
STATUSMAIL="no"

#Rsync over ssh or to rsync daemon (ssh/rsync)
RSYNCSTYLE="rsync"

#Enable Nagios Monitoring (yes/no)
NAGIOSMONITORING="yes"

##########################################################################
##Functions
##########################################################################

function STARTBLABLA {
echo "#############################################"                            >  $LOGFILE
echo "#Starting sync `date +"%k:%M %d.%m.%Y"`:"                                 >> $LOGFILE
echo ""              
}

function RSYNCSERVER {
ssh $RSUSER@$RSTARGET /etc/init.d/rsync start					>> $LOGFILE
RSYNCDAEMONSTART=$?
echo "ErrorCode: $RSYNCDAEMONSTART"						>> $LOGFILE
sleep 5
for i in $BACKUPPATH ; do
	/usr/bin/rsync --delete -avz --partial --inplace $EXCLUDE --log-file=$LOGFILE $i $RSTARGET::$BACKUPTARGET
	RSYNCERROR=$?
	if [ "$RSYNCERROR" != "0" ]; then
		if [ $NAGIOSMONITORING = yes ]; then
			NAGIOS
		fi
		echo "Rsync Failure: $RSYNCERROR"				>> $LOGFILE
		ENDBLABLA
		MAIL
		exit 1
	fi
done
ssh $RSUSER@$RSTARGET /etc/init.d/rsync stop					>> $LOGFILE
RSYNCDAEMONSTOP=$?
echo "ErrorCode: $RSYNCDAEMONSTOP"                                              >> $LOGFILE
}

function RSYNCSSH {
for i in $BACKUPPATH ; do
	/usr/bin/rsync --delete -avz --progress --partial --inplace $EXCLUDE --log-file=$LOGFILE $i $RSUSER@$RSTARGET:$BACKUPTARGET
	RSYNCERROR=$?
	if [ "$RSYNCERROR" != "0" ]; then
		if [ $NAGIOSMONITORING = yes ]; then
			NAGIOS
		fi
		echo "Rsync Failure: $RSYNCERROR"				>> $LOGFILE
		ENDBLABLA
		MAIL
		exit 1
	fi
done
}

function ENDBLABLA {
echo ""                                                                         >> $LOGFILE
echo "#Sync ended `date +"%k:%M %d.%m.%Y"`:"                                    >> $LOGFILE
echo "#############################################"                            >> $LOGFILE
}

function RUNNINGTIME {
# Sekundenzhler stoppen ########################################
ende=$(date +%s)

diff=$[ende-anfang]
echo -e "\n"
RUNTIME="Runtime: $[$diff / 60]min $[$diff % 60]s"
}

function MAIL {
mail -a "From: <$FROM>" -s "`cat /etc/hostname` Backup" $MAILADRESS           < $LOGFILE
}

function NAGIOSRUNNING {
	MESSAGE="RUNNING: Backup is still running"
	echo "`date +%Y%m%d%H%M` $MESSAGE" > $NAGIOSLOGFILE
}

function NAGIOS {
if [ "$RSYNCERROR" != "0" ]; then
	MESSAGE="ERROR: Problems at Rsyncjob. - Errorcode: $RSYNCERROR" - $RUNTIME 
elif [ "$RSYNCERROR" = "0" ]; then
	MESSAGE="SUCCESS: Backup runs well! - $RUNTIME"
else
	MESSAGE="UNKNOWN: Other Error. - $RUNTIME" 
fi
echo "`date +%Y%m%d%H%M` $MESSAGE" > $NAGIOSLOGFILE
}

##########################################################################
##Execute the functions
##########################################################################

# Sekundenzhler starten ########################################
anfang=$(date +%s)

if [ $NAGIOSMONITORING = yes ]; then
        NAGIOSRUNNING
fi
STARTBLABLA
if [ $RSYNCSTYLE = rsync ]; then
	RSYNCSERVER
elif [ $RSYNCSTYLE = ssh ]; then
	RSYNCSSH
fi
ENDBLABLA
RUNNINGTIME
if [ $STATUSMAIL = yes ]; then
	MAIL
fi
if [ $NAGIOSMONITORING = yes ]; then
	NAGIOS
fi
